@@ -388,6 +388,27 @@ pre tag
         assert store.units[0].source == "Extract this"
         assert store.units[1].source == "Translate"
 
+        # Ignored content nested in parent translatable element should not be duplicated
+        # This is the bug from https://github.com/WeblateOrg/weblate/issues/17867
+        store = h.parsestring(
+            """<div>
+  <p>Welcome</p>
+  <ul class="lang-menu" data-translate-ignore>
+    <li><a href="/ar" lang="ar">العربية</a></li>
+    <li><a href="/ca" lang="ca">Català</a></li>
+  </ul>
+  <p>Choose your language</p>
+</div>"""
+        )
+        assert len(store.units) == 2
+        assert store.units[0].source == "Welcome"
+        assert store.units[1].source == "Choose your language"
+        # Ensure the ignored content is NOT in any unit
+        for unit in store.units:
+            assert "العربية" not in unit.source
+            assert "Català" not in unit.source
+            assert "<li>" not in unit.source
+
     def test_translate_comment_directives(self) -> None:
         """Check that translate:off and translate:on comments work."""
         h = html.htmlfile()
@@ -415,6 +436,29 @@ pre tag
         assert len(store.units) == 1
         assert store.units[0].source == "Translate this"
 
+        # Ignored content nested in parent translatable element should not be duplicated
+        # Same test as above but with comment directives instead of attribute
+        store = h.parsestring(
+            """<div>
+  <p>Welcome</p>
+  <!-- translate:off -->
+  <ul class="lang-menu">
+    <li><a href="/ar" lang="ar">العربية</a></li>
+    <li><a href="/ca" lang="ca">Català</a></li>
+  </ul>
+  <!-- translate:on -->
+  <p>Choose your language</p>
+</div>"""
+        )
+        assert len(store.units) == 2
+        assert store.units[0].source == "Welcome"
+        assert store.units[1].source == "Choose your language"
+        # Ensure the ignored content is NOT in any unit
+        for unit in store.units:
+            assert "العربية" not in unit.source
+            assert "Català" not in unit.source
+            assert "<li>" not in unit.source
+
     def test_extraction_meta_social_media_tags(self) -> None:
         """Check that we can extract common social media meta tags."""
         h = html.htmlfile()
@@ -494,6 +538,52 @@ pre tag
         assert store.units[1].source == "Page description"
         assert store.units[2].source == "Twitter Title"
 
+    def test_extraction_meta_additional_og_twitter_tags(self) -> None:
+        """Check that additional Open Graph and Twitter meta tags can be extracted."""
+        h = html.htmlfile()
+
+        # Test og:image:alt and twitter:image:alt
+        store = h.parsestring(
+            """<html><head>
+            <meta property="og:image:alt" content="Description of the image">
+            <meta name="twitter:image:alt" content="Twitter image description">
+            </head><body></body></html>"""
+        )
+        assert len(store.units) == 2
+        assert store.units[0].source == "Description of the image"
+        assert store.units[1].source == "Twitter image description"
+
+        # Test video tags
+        store = h.parsestring(
+            """<html><head>
+            <meta property="video:actor:role" content="Lead Actor">
+            <meta property="video:tag" content="action, thriller">
+            </head><body></body></html>"""
+        )
+        assert len(store.units) == 2
+        assert store.units[0].source == "Lead Actor"
+        assert store.units[1].source == "action, thriller"
+
+        # Test article tags
+        store = h.parsestring(
+            """<html><head>
+            <meta property="article:section" content="Technology">
+            <meta property="article:tag" content="programming, python">
+            </head><body></body></html>"""
+        )
+        assert len(store.units) == 2
+        assert store.units[0].source == "Technology"
+        assert store.units[1].source == "programming, python"
+
+        # Test payment:description
+        store = h.parsestring(
+            """<html><head>
+            <meta property="payment:description" content="Premium subscription">
+            </head><body></body></html>"""
+        )
+        assert len(store.units) == 1
+        assert store.units[0].source == "Premium subscription"
+
     def test_data_translate_comment_attribute(self) -> None:
         """Check that data-translate-comment attribute is extracted as a note."""
         h = html.htmlfile()
@@ -538,3 +628,76 @@ pre tag
         assert store.units[0].source == "<strong>Bold</strong> text here"
         notes = store.units[0].getnotes(origin="source code")
         assert notes == "This is important"
+
+
+class TestHTMLDocpath:
+    h = html.htmlfile
+
+    def test_basic_structure(self) -> None:
+        """Test docpath for basic HTML structure."""
+        store = self.h.parsestring(
+            "<html><body><h1>Title</h1><p>First</p><p>Second</p></body></html>"
+        )
+        docpaths = [u.getdocpath() for u in store.units]
+        assert docpaths == [
+            "html[1]/body[1]/h1[1]",
+            "html[1]/body[1]/p[1]",
+            "html[1]/body[1]/p[2]",
+        ]
+
+    def test_nested_structure(self) -> None:
+        """Test docpath with nested sections."""
+        store = self.h.parsestring(
+            "<html><body>"
+            "<h1>Chapter</h1><p>Intro</p>"
+            "<div><p>Nested</p></div>"
+            "</body></html>"
+        )
+        docpaths = [u.getdocpath() for u in store.units]
+        assert docpaths == [
+            "html[1]/body[1]/h1[1]",
+            "html[1]/body[1]/p[1]",
+            "html[1]/body[1]/div[1]/p[1]",
+        ]
+
+    def test_multiple_headings(self) -> None:
+        """Test docpath with multiple heading levels."""
+        store = self.h.parsestring(
+            "<html><body>"
+            "<h1>Title</h1>"
+            "<h2>Sub1</h2>"
+            "<p>Content1</p>"
+            "<h2>Sub2</h2>"
+            "<p>Content2</p>"
+            "</body></html>"
+        )
+        docpaths = [u.getdocpath() for u in store.units]
+        assert docpaths == [
+            "html[1]/body[1]/h1[1]",
+            "html[1]/body[1]/h2[1]",
+            "html[1]/body[1]/p[1]",
+            "html[1]/body[1]/h2[2]",
+            "html[1]/body[1]/p[2]",
+        ]
+
+    def test_list_items(self) -> None:
+        """Test docpath for list items."""
+        store = self.h.parsestring(
+            "<html><body><ul><li>One</li><li>Two</li></ul></body></html>"
+        )
+        docpaths = [u.getdocpath() for u in store.units]
+        assert docpaths == [
+            "html[1]/body[1]/ul[1]/li[1]",
+            "html[1]/body[1]/ul[1]/li[2]",
+        ]
+
+    def test_docpath_stable_across_content_changes(self) -> None:
+        """Test that docpath is the same for structurally equivalent documents."""
+        store1 = self.h.parsestring("<html><body><h1>Short</h1><p>A</p></body></html>")
+        store2 = self.h.parsestring(
+            "<html><body><h1>A much longer title here</h1>"
+            "<p>A much longer paragraph with more content</p></body></html>"
+        )
+        assert [u.getdocpath() for u in store1.units] == [
+            u.getdocpath() for u in store2.units
+        ]
