@@ -47,23 +47,23 @@ class TestXLIFF2Unit(test_base.TestTranslationUnit):
         assert unit.istranslatable()
         unit.marktranslatable(False)
         assert not unit.istranslatable()
-        assert unit.getunitelement().get("translate") == "no"  # ty:ignore[possibly-missing-attribute]
+        assert unit.getunitelement().get("translate") == "no"  # ty:ignore[unresolved-attribute]
 
         # Test marking as translatable
         unit.marktranslatable(True)
         assert unit.istranslatable()
-        assert unit.getunitelement().get("translate") == "yes"  # ty:ignore[possibly-missing-attribute]
+        assert unit.getunitelement().get("translate") == "yes"  # ty:ignore[unresolved-attribute]
 
         # Test that marking as translatable again when already translatable sets to "yes"
         unit.marktranslatable(True)
         assert unit.istranslatable()
-        assert unit.getunitelement().get("translate") == "yes"  # ty:ignore[possibly-missing-attribute]
+        assert unit.getunitelement().get("translate") == "yes"  # ty:ignore[unresolved-attribute]
 
         # Test that marking as untranslatable when already untranslatable doesn't change it
         unit.marktranslatable(False)
         unit.marktranslatable(False)
         assert not unit.istranslatable()
-        assert unit.getunitelement().get("translate") == "no"  # ty:ignore[possibly-missing-attribute]
+        assert unit.getunitelement().get("translate") == "no"  # ty:ignore[unresolved-attribute]
 
 
 class TestXLIFF2file(test_base.TestTranslationStore):
@@ -588,3 +588,79 @@ class TestXLIFF2file(test_base.TestTranslationStore):
         assert bytes(xliff2file).decode() == xliff2_content.decode().replace(
             "final", "reviewed"
         )
+
+    def test_xliff2_rejects_xliff1_files(self) -> None:
+        """Test that XLIFF 2.0 parser rejects XLIFF 1.x files with a clear error."""
+        # Test XLIFF 1.1
+        xliff11_content = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="test.txt" source-language="en" datatype="plaintext">
+        <body>
+            <trans-unit id="1">
+                <source>Hello</source>
+                <target>Hola</target>
+            </trans-unit>
+        </body>
+    </file>
+</xliff>"""
+        with pytest.raises(ValueError) as exc_info:
+            xliff2.Xliff2File.parsestring(xliff11_content)
+        assert "XLIFF 1" in str(exc_info.value)
+        assert "version='1.1'" in str(exc_info.value)
+        assert "xliff.xlifffile" in str(exc_info.value)
+
+        # Test XLIFF 1.2
+        xliff12_content = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.2" xmlns="urn:oasis:names:tc:xliff:document:1.2">
+    <file original="test.txt" source-language="en" datatype="plaintext">
+        <body>
+            <trans-unit id="1">
+                <source>Hello</source>
+                <target>Hola</target>
+            </trans-unit>
+        </body>
+    </file>
+</xliff>"""
+        with pytest.raises(ValueError) as exc_info:
+            xliff2.Xliff2File.parsestring(xliff12_content)
+        assert "XLIFF 1" in str(exc_info.value)
+        assert "version='1.2'" in str(exc_info.value)
+        assert "xliff.xlifffile" in str(exc_info.value)
+
+    def test_xliff2_rejects_xliff1_files_without_version(self) -> None:
+        """Test that XLIFF 2.0 parser rejects XLIFF 1.x files without version attribute."""
+        # Test XLIFF 1.1 without version
+        xliff11_no_version = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="test.txt" source-language="en" datatype="plaintext">
+        <body>
+            <trans-unit id="1">
+                <source>Hello</source>
+                <target>Hola</target>
+            </trans-unit>
+        </body>
+    </file>
+</xliff>"""
+        with pytest.raises(ValueError) as exc_info:
+            xliff2.Xliff2File.parsestring(xliff11_no_version)
+        assert "XLIFF 1" in str(exc_info.value)
+        assert "namespace" in str(exc_info.value)
+        assert "xliff.xlifffile" in str(exc_info.value)
+
+        # Test XLIFF 1.2 without version
+        xliff12_no_version = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff xmlns="urn:oasis:names:tc:xliff:document:1.2">
+    <file original="test.txt" source-language="en" datatype="plaintext">
+        <body>
+            <trans-unit id="1">
+                <source>Hello</source>
+                <target>Hola</target>
+            </trans-unit>
+        </body>
+    </file>
+</xliff>"""
+        with pytest.raises(ValueError) as exc_info:
+            xliff2.Xliff2File.parsestring(xliff12_no_version)
+        assert "XLIFF 1" in str(exc_info.value)
+        assert "namespace" in str(exc_info.value)
+        assert "xliff.xlifffile" in str(exc_info.value)
